# -*- coding: UTF-8 -*-
"""
Prevod TIFF + TFW snímok s RGB a NIR kanálom na samostatnú RGB snímku a NIR snímku.
Nástroj vznikol prioritne pre dávkové spracovanie verejne dostupných snímok ortofotomozaiky SR (2. cyklus),
ktorú dodáva GKÚ Bratislava (stiahnutie na https://www.geoportal.sk/sk/udaje/ortofotomozaika/)
Dodávané súbory z GKÚ Bratislava sú vo formáte TIFF + TFW, Súradnicový systém: S-JTSK(JTSK) - kód EPSG:5514

Nástroj spracováva dávkovo všetky zdrojové snímky vo formáte TIFF + TFW vo vybranom priečinku.
Použitie nástroja predpokladá, že všetky zdrojové snímky su v súradnicovom systéme S-JTSK(JTSK) - kód EPSG:5514

Po spustení nástroj vyzve uživateľa, aby vybral priečinok so zdrojovými snímkami vo formáte TIFF + TFW.
Následne nástroj vyzve uživateľa, aby zadal cieľový priečinok, kam budú spracované snímky uložené v GeoTIFF použiteĺné aj s TFW súborom .

Nástroj neupravuje zdrojové snimky, ale vytvára nové snímky vo formáte GeoTIFF v súradnicovom systéme S-JTSK(JTSK) - kód EPSG:5514 v pôvodnom rozlíšení s pôvodným názvom snímky.

Autor: Jakub Fuska, ATAPEX s. r. o., www.atapex.sk, atapex@atapex.sk

UPOZORNENIE: Autor nenesie zodpovednosť za výsledky použitia nástroja.
"""

import os, sys, datetime, shutil
from osgeo import gdal
from PyQt5.QtWidgets import QFileDialog
from PyQt5.QtWidgets import QInputDialog


options_list = [
    '-co tfw=yes',
]

options_string = " ".join(options_list)

#Prázdna premenná, kam sa uloží názov prvého nájdeného TIF súboru
prve_tif =''
 
#Otvorí dialóg pre výber zdrojového priečinku s originálnymi snímkami
vstupny_priecinok = str(QFileDialog.getExistingDirectory(None, 'Zadaj priecinok so vstupnymi snimkami'))

print ('Priecinok so zdrojovymi datami bol zvoleny: '+ vstupny_priecinok)
#nastavenie oddeľovača
oddelovac = vstupny_priecinok[2]

#Otvorí dialóg pre výber cieľového priečinku, kde budu uložené spracované snímky
cielovy_priecinok = str(QFileDialog.getExistingDirectory(None, 'Vyber priecinok, kam maju byt ulozene spracovane snimky'))
print ('Vysledok spracovania bude ulozeny v priecinku: '+ cielovy_priecinok)
print('--------')
#počítadlo súborov vo vstupnom priečinku
pocitadlo_tif = 0

#Prejde vstupný priečinok a zistí, či tento priečinok obsahuje TIF súbory
for subor in os.listdir(vstupny_priecinok):
    if subor[-3:] =='tif':
        #Zapíše názov prvého nájdeného TIF súboru
        prve_tif = os.listdir(vstupny_priecinok)[pocitadlo_tif]        
        break
    pocitadlo_tif +=1


if prve_tif =='':
    #Hlásenie, ak sa vo vstupnom priečinku nenachádza žiadny TIF súbor
    print('Vstupny priecinok \"' + vstupny_priecinok +'\" neobsahuje ziadne TIF subory')
    
else:
    print('Zacinam pracovat...')
    #vytvorenie dočasného podpriečinku
    docasny_priecinok = cielovy_priecinok + oddelovac + 'temp'
    os.mkdir(docasny_priecinok)
    
    #vytvorenie dočasného podpriečinku s virtuálnym RGB rastrom
    rgb_docasny_priecinok = docasny_priecinok + oddelovac + 'rgb'
    os.mkdir(rgb_docasny_priecinok)
    
    #vytvorenie dočasného podpriečinku s virtuálnym NIR rastrom
    nir_docasny_priecinok = docasny_priecinok + oddelovac + 'nir'
    os.mkdir(nir_docasny_priecinok)
    
    #vytvorenie cieľového podpriečinku s výsledným RGB rastrom
    rgb_cielovy_priecinok = cielovy_priecinok + oddelovac + 'rgb'
    os.mkdir(rgb_cielovy_priecinok)
    
    #vytvorenie podpriečinku pre TFW súbory výsledných RGB rastrov
    rgb_tfw_podpriecinok = rgb_cielovy_priecinok + oddelovac + 'rgb_tfw'
    os.mkdir(rgb_tfw_podpriecinok)
    
    #vytvorenie cieľového podpriečinku s výsledným NIR rastrom
    nir_cielovy_priecinok = cielovy_priecinok + oddelovac + 'nir'
    os.mkdir(nir_cielovy_priecinok)
    
    #vytvorenie podpriečinku pre TFW súbory výsledných NIR rastrov
    nir_tfw_podpriecinok = nir_cielovy_priecinok + oddelovac + 'nir_tfw'
    os.mkdir(nir_tfw_podpriecinok)
    
    #Dialóg pre rozšírenie názvu súboru výsledného RGB rastra (pri zadaní prázdnej hodnoty nebude rozšírený názov výsledného RGB rastra)
    rozsirenie_nazvu_rgb = QInputDialog.getText(None, 'RGB', 'Zadaj text na rozsirenie nazvu vyslednych RGB snimok')[0]
    
    #Dialóg pre rozšírenie názvu súboru výsledného NIR rastra  (pri zadaní prázdnej hodnoty nebude rozšírený názov výsledného NIR rastra)
    rozsirenie_nazvu_nir = QInputDialog.getText(None, 'NIR', 'Zadaj text na rozsirenie nazvu vyslednych NIR snimok')[0]
    
    """
    Cyklus na prejdenie obsahu vstupného priečinku a uloženie GeoTIFF súboru v súradnicovom systéme rovnakom ako vo vstupnom súbore 
    a uloženie pásiem 1 (red/červené), 2 (green/zelené) a 3 (blue/modré) do GeoTIFF v podpriečinku RGB
    a uloženie pásma 4 (NIR) do GeoTIFF v podpriečinku NIR.
    Vytvorené RGB a NIR snímky budú mať zadefinovaný súradnicový systém EPSG: 5514 a takisto bude vytvorený aj TFW súbor,
    ktorý môže, ale nemusí byť použitý - súbory TFW budú pre RGM aj NIR snímky uložené v samostatnom podpriečinku. 
    """
    
    for subor in os.listdir(vstupny_priecinok):
        if subor[-3:] == 'tif':
            print('Spracovavam subor: ' + vstupny_priecinok+oddelovac+subor)
            vstupny_raster = gdal.Open(vstupny_priecinok+oddelovac+subor)
            virtual_rgb_subor = rgb_docasny_priecinok+oddelovac+subor
            #vytvorenie virtuálneho RGB rastra
            gdal.BuildVRT(virtual_rgb_subor,
                   vstupny_raster,bandList=[1,2,3]
                   )
            #vytvorenie virtuálneho NIR rastra
            virtual_nir_subor = nir_docasny_priecinok+oddelovac+subor
            gdal.BuildVRT(virtual_nir_subor,
                   vstupny_raster,bandList=[4]
                   )
     

            #vytvorenie výsledného RGB rastra
            rgb_vrstva = rgb_cielovy_priecinok+oddelovac+subor[:-4] + rozsirenie_nazvu_rgb  +subor[-4:]
            rgb_tfw = rgb_cielovy_priecinok+oddelovac+subor[:-4] + rozsirenie_nazvu_rgb  +'.tfw'
            gdal.Translate(rgb_vrstva,
                   virtual_rgb_subor,
                   options=options_string
                   )
            
            shutil.move(rgb_tfw, rgb_tfw_podpriecinok+oddelovac+subor[:-4] + rozsirenie_nazvu_rgb  +'.tfw') 
            
            os.remove(rgb_vrstva)
            gdal.Warp(rgb_vrstva, virtual_rgb_subor, dstSRS = "EPSG:5514")
            
            print('Vysledny RGB subor ' + rgb_vrstva + ' bol zapisany')
            
            #vytvorenie výsledného NIR rastra
            nir_vrstva = nir_cielovy_priecinok+oddelovac+subor[:-4] + rozsirenie_nazvu_nir  +subor[-4:]
            nir_tfw = nir_cielovy_priecinok+oddelovac+subor[:-4] + rozsirenie_nazvu_nir  +'.tfw'
            gdal.Translate(nir_vrstva,
                   virtual_nir_subor,
                   options=options_string
                   )
            
            shutil.move(nir_tfw, nir_tfw_podpriecinok+oddelovac+subor[:-4] + rozsirenie_nazvu_nir  +'.tfw') 
            
            #uprava do geotiff
            os.remove(nir_vrstva)
            gdal.Warp(nir_vrstva, virtual_nir_subor, dstSRS = "EPSG:5514")
            
            print('Vysledny NIR subor ' + nir_vrstva + ' bol zapisany')
            
    #Zmazanie dočasného podpriečinku vrátane podpriečinkov a súborov virtuálnych RGB a NIR rastrov
    shutil.rmtree(docasny_priecinok)
print('--------')
print('Hotovo')
